
chat.print("**************************************************************************");
chat.print("Welcome to the test pickup script. ");
chat.print("Pick up range set to 20 feet");
chat.print("Click on an object when within that range to pick it up");
chat.print("Click again on an object to release it");
chat.print("**************************************************************************");
chat.print("");

/************************************************************************************
 * Object pick up script
 *
 * Generic script to pick and up do stuff with models within the world. 
 * "Picking up" an object in this context is being able to click on an 
 * object and have it appear "with" the actor as he/she is moving. 
 * The actor can then choose to release the object by clicking on it again
 * 
 * You may enable Synchronisation using Shared Objects by setting a variable (see
 * below) "usesSynchObjects = true". 
 ************************************************************************************/

//===============================================================================
// boxarr - set the array to the filenames of models you'd like to enable pickup
//===============================================================================
boxarr = ["./cleanerbot.aer", "./cleanerbot.aer", "./cleanerbot.aer"];

//===============================================================================
// enableloadinorder - forces the boxarr models to be appear loaded in the order 
//              in which they appear in the array. Ie. boxarr[0] loads first, etc.
//===============================================================================
enableloadinorder = true;

//===============================================================================
// pickuprange - the actor must be within this range (in feet) to be able to 
//               pick the object up
//===============================================================================
pickuprange = 20;

//===============================================================================
// closeness - determined how close the picked up object appears to the actor
//             Eg. 1.0 == 1 Foot away.
//===============================================================================
closeness = 2.0;


//===============================================================================
// boxmodels - the final models are stored in this array. No need to modify this.
//===============================================================================

boxmodels = [];

//===============================================================================
// usesSynchObjects - synchronises pickup & release of objects with other actors
//                    Good for multi-users. 
//===============================================================================
usesSynchObjects = true;

// NB: you may also dynamically enable Synchronisation on your side by calling the function:
// "enableSynchObjects(<true/false>);"


//===============================================================================
// Regulates the frequency of update messages send to other users. 
// Default is 1.0 seconds. This means synchronisation will occur every 1.0 seconds
//===============================================================================
synchroniseFrequency = 1.0;


//===============================================================================
// Call-back functions!
// 
// Place code within these functions. They will get called at the right time
//===============================================================================

function onmodelload(model, index) {

// When a model loads, this function will get called & you can do something to it. 
// For example: setting the positions of the models, creating physical models
// etc.

	// Physical Model added with some gravity
	model.pm = model.createConvexPhysicalModel(10,0);
	model.pm.fixed = false;
	model.pm.collide = true;
	model.pm.acceleration = [0,-10,0];
	
	if(index==0) {
		// The FIRST cleanerbot	
		model.position = [10,0,10];
	} else if(index==1) {
		// The SECOND cleanerbot
		model.position = [-10,0,10];
	} else {
		model.position = [10,0,-10];	
	}
	chat.print("Model has loaded (boxarr["+index+"])");

}


function onrelease(model, index) {

// What you want to do with the model when you release it. 
// For example: you might want to set a new position

	model.pm.collide = true;
	model.pm.acceleration = [0,-10,0];

	chat.print("Model has been released (boxarr["+index+"])");

}

function onpickup(model, index) {

// What you want to do with the model once you pick it up
// For example: you might want to disable a physical model

	model.pm.collide = false;
	model.pm.acceleration = [0,0,0];

	chat.print("Model has been picked up (boxarr["+index+"])");

}



function onExternalUserPickup(model, index) {

	model.pm.collide = false;
	model.pm.acceleration = [0,0,0];

	chat.print("Another user has picked up (boxarr["+index+"])");
}

function onExternalUserRelease(model, index) {

	model.pm.collide = true;
	model.pm.acceleration = [0,-10,0];

	chat.print("Another user has released (boxarr["+index+"])");

}


closeslider = Slider("Closeness").add();
closeslider.value = 2.0;
closeslider.range = [0.5, 5.0];
closeslider.onChange = function(val) {

closeness = val;
}

synchenable = Toggle("Synchronisation").add();
synchenable.enabled = true;
synchenable.state = true;
synchenable.onClick = function(enable) {
	synchenable.state = enable;
	enableSynchObjects(enable);
}


//***********************************************************************************

actoriscarrying = undefined;

//***********************************************************************************
// There is no need to alter code beyond this point
//***********************************************************************************

obj = Object();
obj.main = function() {
	obj.loadindex = 0;
	
	
	// Adds a model to the "boxmodels" array
	obj.addModel = function(name) {
		for (i=0; i<boxmodels.length; i++) {
			if (boxmodels[i] == false) {
				boxmodels[i] = name;
				return;
			}
		}
	
		boxmodels[boxmodels.length] = name;
	}

	// Removes a model from the "boxmodels" array
	obj.removeModel = function(name) {
		for(i=0; i<boxmodels.length; i++) {
			if (boxmodels[i] == name) {
				boxmodels[i] = false;
				return;
			}
		}
	}
	
	
	// Recursively called function
	// obj.loadbox loads a single box when it is supposed to 
	obj.loadbox = function() {
		var box=Model(boxarr[obj.loadindex]); 
		box._index = obj.loadindex++;
		box.visible = false;
		box.add();
		box._carry = false;
		box._otherlocked = false;
		box._playerlocked = false;
		
		// Our onclick mouse event handler :)
		box._mouse = MouseEventHandler();
		box._mouse.setFilterOnClick(box);
		box._mouse.what=box;
		
		box.onLoad = function() {
		
			// Run the call back function 
			onmodelload(this, this._index);
			this.visible = true;
			
			// This is called when the user wants to force model load order
			if(enableloadinorder) {
				if(obj.loadindex<boxarr.length) 
				obj.loadbox();
			}
			this._mouse.onEvent = function(event) {			
				if(actoriscarrying==undefined && (actor.dist(this.what.position)<pickuprange)) {
					if(!usesSynchObjects || (usesSynchObjects && this.what._otherlocked == false)) {
					actoriscarrying = this.what;
					this.what._carry = true;
					addAnimator(obj);
					onpickup(this.what,this.what._index);
					if(usesSynchObjects) {
						obj.carryObject(this.what, true);
					}	
					}
				} else if(actoriscarrying==this.what) {
					this.what._carry = false;
					actoriscarrying=undefined;
					onrelease(this.what,this.what._index); 
					removeAnimator(obj);
					if(usesSynchObjects) {
						obj.carryObject(this.what, false);
					}	
				}	
			}
			obj.addModel(this);
			
			removeAnimator(box);
		}
		addAnimator(box);	
		
		// This is called when the user doesn't want to force model load order
		if(!enableloadinorder) {
			if(obj.loadindex<boxarr.length) 
			obj.loadbox();
		}
	}
	
	obj.loadbox();

}


// This timestep sets the Models position with the Actors position, 
// so it appears that the 2 are moving together as if the Actor is 
// carrying the model
// 

obj.timestep=function(){ 
	if(actoriscarrying!=undefined){ 
		actoriscarrying.position = actor.position.add(actor.facing.scale(closeness)).subtract(Vector(0,1,0));
// To fix the Models rotation, uncomment the following line: 
		actoriscarrying.orientation = actor.transform.rotation;
	}
}

obj.main();


//**************************************************************************************
// Synchronize using Shared Objects
//**************************************************************************************
shareObjName = "pickupmodels";

obj.enableSynchObjects = function(isenabled) {
		
	// If no synchronisation is enabled, disable all the functions. 
	// Perhaps undefining onSharedProperty change speeds things up?
	if(!isenabled) {
		usesSynchObjects = false;
		obj.carryObject = undefined;
		chat.onSharedPropertyChange = undefined;
		return;
	}
		
	// If we get here, we know that Synchronisation is being enabled
	usesSynchObjects=true;

	// This function is called whenever the main Actor in the world either picks or 
	// releases an object/model. 
	obj.carryObject = function(model, ispickedup) {
		if(chat.isConnected) {

			if(model.timestep==undefined) {
				// Create the model synchronisation updated timestep 
					model.onLoad = undefined;
				model._stoptimestep = true;
				model._tottime = 0;
				model.timestep = function(w,d) {
					if(this._stoptimestep) {
						model._tottime = 0;
						removeAnimator(model);
						chat.setSharedProperty(shareObjName, ""+this._index, "null");				
					} else if((model._tottime+=d) > synchroniseFrequency) {
						model._tottime = 0;
						chat.setSharedProperty(shareObjName, ""+this._index, this.position.toString());
					}
				}
			}

			if(ispickedup) {
				model._playerlocked = true;
				model._stoptimestep = false;
				addAnimator(model);	
			} else {
				model._stoptimestep = true;
			}
		}
	}

	// onSharedPropertyChange is called whenever an objects property has changed
	// In this case, a Model being picked up/released will update to this method
	// We want to know when this happens in order to reflect the Models
	// updated movements
	// 
	// Important note: only one Shared Object is defined by this script: "pickupmodels"
	// on this Shared Object, a property for each Model is add + the postion of the model. 
	// Eg.    SharedObject = "pickupmodels"
	//        Property     = "0":"V[0,0,0]"
	//        Property     = "1":"V[5.212,0.22,-0.242]"

	chat.onSharedPropertyChange = function(objname, property, value) {
//chat.print(objname+" "+property+" "+value+" ");

	// First IF -- when an Object is created, we want to check whether any properties
	// already exist for it by calling "requestSharedProperty"
		if(property=="") {
			// Object is newly created
			for(i=0;i<boxarr.length;i++) {
				chat.requestSharedProperty(objname, ""+i);
			}

		} else if(value=="") {
	// A value will only ever equal "" when the property is empty & has not been defined
	// In other words - we are the first person in the world!
	// Thus, we define the property as "null"
			chat.setSharedProperty(objname, property, "null");

		} else if(value=="null") {
	 // When a value has been set to "null" for a property/Model, then we know that
	 // the Model is stationary & is not being 'held' by anyone in the world
			var mod = boxmodels[eval(property)];
			mod._playerlocked = false;
			if(mod._otherlocked) onExternalUserRelease(mod,mod._index);
			mod._otherlocked= false;
			mod._stoptimestep = false;
		} else {
	// In all other cases, our value has a Vector location. This will come abouts from 
	// either our Actor, or another Actor in the world
			var mod = boxmodels[eval(property)];

			if(mod._stoptimestep) {
				// We are the Actor & we've released the Model. Thus we don't want to 
				// let any position updated in.. 
					
			} else if(!mod._playerlocked) { 
				// Indicates another Actor in the world has picked the Model up.
//				mod.position = Vector(value);
				mod.suggestLocation(Transform(Vector(value),mod.orientation));

				if(!mod._otherlocked) onExternalUserPickup(mod, mod._index);				
				mod._otherlocked=true;
			}
		}
	}


	// Timestep to check whether the chat is connected
	// chat.onConnect callback is too unreliable for when is connect before the AJS
	// has loaded
	chatloader = Object();
	chatloader.timestep = function() {
		if(chat.isConnected) {
			var oerrer;
			try { 
				chat.createSharedObject("pickupmodels");
			} catch (oerrer) {
				chat.print("Create SharedObj - ",oerror.name,": ",oerror.message);
			}
			removeAnimator(this);
		}
	}
	addAnimator(chatloader);

}

function enableSynchObjects(enable) {
	obj.enableSynchObjects(enable);
}

enableSynchObjects(usesSynchObjects);
